/**************************************************************************
*   Amarok 2 script to manage lyrics                                      *
*                                                                         *
*   Copyright                                                             *
*   (C) 2008 Ignacio Serantes <kde@aynoa.net>                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.         *
**************************************************************************/


function configurationEditor() {

  this.dlg = new QWidget(this);

  this.deleteCacheLyrics = function() {

    var path = PATH_LYRICS;
    removeDirectory(Amarok.Info.scriptPath() + '/' + path);
    if (!scriptPath.exists(path)) { scriptPath.mkdir(path); }
    this.btDeleteCacheLyrics.toolTip = this.calculateCacheSize(path);

  } // this.deleteCacheLyrics = function()

  this.deleteCacheTranslations = function() {

    var path = PATH_TRANSLATIONS;
    removeDirectory(Amarok.Info.scriptPath() + '/' + path);
    if (!scriptPath.exists(path)) { scriptPath.mkdir(path); }
    this.btDeleteCacheTranslations.toolTip = this.calculateCacheSize(path);

  } // this.deleteCacheTranslation = function()

  this.deleteCacheRomaji = function() {

    var path = PATH_ROMAJI;
    removeDirectory(Amarok.Info.scriptPath() + '/' + path);
    if (!scriptPath.exists(path)) { scriptPath.mkdir(path); }
    this.btDeleteCacheRomaji.toolTip = this.calculateCacheSize(path);

  } // this.deleteCacheRomaji = function()

  this.deleteCacheKana = function() {

    var path = PATH_KANA;
    removeDirectory(Amarok.Info.scriptPath() + '/' + path);
    if (!scriptPath.exists(path)) { scriptPath.mkdir(path); }
    this.btDeleteCacheKana.toolTip = this.calculateCacheSize(path);

  } // this.deleteCacheKana = function()

  this.deleteCacheRomaja = function() {

    var path = PATH_ROMAJA;
    removeDirectory(Amarok.Info.scriptPath() + '/' + path);
    if (!scriptPath.exists(path)) { scriptPath.mkdir(path); }
    this.btDeleteCacheRomaja.toolTip = this.calculateCacheSize(path);

  } // this.deleteCacheRomaja = function()

  this.deleteCacheKorean = function() {

    var path = PATH_KOREAN;
    removeDirectory(Amarok.Info.scriptPath() + '/' + path);
    if (!scriptPath.exists(path)) { scriptPath.mkdir(path); }
    this.btDeleteCacheKorean.toolTip = this.calculateCacheSize(path);

  } // this.deleteCacheKorean = function()

  this.accept = function() {

    this.saveConfiguration();
    this.dlg.close();

  } // this.accept = function()

  this.close = function() {

    this.dlg.close();

  } // this.close = function()

  this.show = function() {

    this.dlg.show();

  } // this.show = function()

  this.saveConfiguration = function() {

    try {

      cfg[CFG_CACHE_KANA] = this.ckCacheKana.checked;
      cfg[CFG_CACHE_KOREAN] = true; //this.ckCacheKorean.checked;
      cfg[CFG_CACHE_LYRICS] = this.ckCacheLyrics.checked;
      cfg[CFG_CACHE_ROMAJA] = this.ckCacheRomaja.checked;
      cfg[CFG_CACHE_ROMAJI] = this.ckCacheRomaji.checked;
      cfg[CFG_CACHE_TRANSLATION] = this.ckCacheTranslations.checked;
      cfg[CFG_EXTERNAL_LYRICS_SHOW] = false;
      cfg[CFG_TRANSLATION_LANGUAGE] = this.cbLanguage.currentText;
      cfg[CFG_REPEAT_SEARCH] = this.ckRepeatSearch.checked;
      cfg[CFG_REPEAT_SEARCH_INCOLLECTION_ONLY] = this.ckRepeatSearchLocalOnly.checked;
      cfg[CFG_SHOW_METHOD] = this.cbShowMethod.currentIndex;
      cfg[CFG_CONVERSION_KANA_SERVER] = this.cbToKanaServer.currentIndex;
      cfg[CFG_CONVERSION_ROMAJI_SERVER] = this.cbToRomajiServer.currentIndex;

      // Creamos el fichero.
      var fs = new QFile(Amarok.Info.scriptPath() + '/' + cfg[CFG_FILE_NAME]);
      fs.open(new QIODevice.OpenMode(QIODevice.Truncate | QIODevice.WriteOnly | QIODevice.Text));
      var ts = new QTextStream(fs);

      ts.writeString('# Lyrics Workshop configuration file\n# ----------------------------------\n');

      ts.writeString('Cache = ' + ((cfg[CFG_CACHE_KANA]) ? '1' : '0') + ((cfg[CFG_CACHE_LYRICS]) ? '1' : '0') + ((cfg[CFG_CACHE_ROMAJA]) ? '1' : '0') + ((cfg[CFG_CACHE_ROMAJI]) ? '1' : '0') + ((cfg[CFG_CACHE_TRANSLATION]) ? '1' : '0') + ((false) ? '1' : '0') + ((false) ? '1' : '0') + ((false) ? '1' : '0') + '\n');

      ts.writeString('ExternalShow = ' + ((cfg[CFG_EXTERNAL_LYRICS_SHOW]) ? '1' : '0') + '\n');
      ts.writeString('RepeatSearch = ' + ((cfg[CFG_REPEAT_SEARCH]) ? '1' : '0') + '\n');
      ts.writeString('RepeatSearchInCollectionOnly = ' + ((cfg[CFG_REPEAT_SEARCH_INCOLLECTION_ONLY]) ? '1' : '0') + '\n');
      ts.writeString('ShowMethod = ' + cfg[CFG_SHOW_METHOD] + '\n');
      ts.writeString('Translate = ' + cfg[CFG_TRANSLATION_LANGUAGE] + '\n');
      ts.writeString('toKanaServer = ' + cfg[CFG_CONVERSION_KANA_SERVER] + '\n');
      ts.writeString('toRomajiServer = ' + cfg[CFG_CONVERSION_ROMAJI_SERVER] + '\n');

      fs.flush();
      fs.close();

    } catch(err) {
      Amarok.debug('error: ' + err);
    }

    return true;

  } // this.saveConfiguration = function()

  this.calculateCacheSize = function(path) {

    if ( path == null) { return ''; }

    path = Amarok.Info.scriptPath() + '/' + path;

    var dir = new QDir(path);
    if (!dir.exists()) { return ''; }
    var list = dir.entryInfoList();
    var size = 0;
    var filesCount = 0
    for (var i = 0; i < dir.count(); i++) {
      if (list[i].isFile()) {
        size += list[i].size();
        filesCount++;
      }
    } // for (var i = 0; i <dir.count(); i++)

    //return format('{1} K, {2} files', Math.round(size/1024), filesCount);
    return Math.round(size/1024) + ' K, ' + filesCount + ' ' + TEXT_CONFIGURE_DIALOG_FILES;

  } // this.calculateCacheSize = function(path)

  this.openCacheLyrics = function() {
    QProcess.execute('kfmclient openURL "' + Amarok.Info.scriptPath() + '/' + PATH_LYRICS + '" "inode/directory"');
  } // this.openCacheLyrics = function()

  this.openCacheTranslations = function() {
    QProcess.execute('kfmclient openURL "' + Amarok.Info.scriptPath() + '/' + PATH_TRANSLATIONS + '" "inode/directory"');
  } // this.openCacheTranslations = function()

  this.openCacheRomaji = function() {
    QProcess.execute('kfmclient openURL "' + Amarok.Info.scriptPath() + '/' + PATH_ROMAJI + '" "inode/directory"');
  } // this.openCacheRomaji = function()

  this.openCacheKana = function() {
    QProcess.execute('kfmclient openURL "' + Amarok.Info.scriptPath() + '/' + PATH_KANA + '" "inode/directory"');
  } // this.openCacheKana = function()

  this.openCacheRomaja = function() {
    QProcess.execute('kfmclient openURL "' + Amarok.Info.scriptPath() + '/' + PATH_ROMAJA + '" "inode/directory"');
  } // this.openCacheRomaja = function()

  this.openCacheKorean = function() {
    QProcess.execute('kfmclient openURL "' + Amarok.Info.scriptPath() + '/' + PATH_KOREAN + '" "inode/directory"');
  } // this.openCacheKorean = function()

  this.ckRepeatSearch_stateChanged = function(state) {
    this.ckRepeatSearchLocalOnly.enabled = (state == Qt.Checked);
  } // this.ckRepeatSearch_stateChanged = function()

  this.grFields = new QGroupBox(TEXT_CONFIGURE_DIALOG_GRFIELDS, this.dlg);
  this.glFields = new QGridLayout(this.dlg);

  this.ckRepeatSearch = new QCheckBox(TEXT_CONFIGURE_DIALOG_CBREPEAT, this.dlg);
  this.ckRepeatSearchLocalOnly = new QCheckBox(TEXT_CONFIGURE_DIALOG_CBREPEATINCOLLECTIONONLY, this.dlg);

  this.ckCacheLyrics = new QCheckBox(TEXT_CONFIGURE_DIALOG_CBCACHELYRICS, this.dlg);
  this.btDeleteCacheLyrics = new QPushButton(new QIcon(Amarok.Info.scriptPath() + '/icons/erase.png'), TEXT_CONFIGURE_DIALOG_BTDELETE_CACHE, this.dlg);
  this.btDeleteCacheLyrics.clicked.connect(this, this.deleteCacheLyrics);
  this.btDeleteCacheLyrics.toolTip = this.calculateCacheSize(PATH_LYRICS);
  this.btOpenCacheLyrics = new QPushButton(new QIcon(Amarok.Info.scriptPath() + '/icons/folder.png'), '', this.dlg);
  this.btOpenCacheLyrics.clicked.connect(this, this.openCacheLyrics);

  this.ckCacheTranslations = new QCheckBox(TEXT_CONFIGURE_DIALOG_CBCACHETRANSLATION, this.dlg);
  this.btDeleteCacheTranslations = new QPushButton(new QIcon(Amarok.Info.scriptPath() + '/icons/erase.png'), TEXT_CONFIGURE_DIALOG_BTDELETE_CACHE, this.dlg);
  this.btDeleteCacheTranslations.clicked.connect(this, this.deleteCacheTranslations);
  this.btDeleteCacheTranslations.toolTip = this.calculateCacheSize(PATH_TRANSLATIONS);
  this.btOpenCacheTranslations = new QPushButton(new QIcon(Amarok.Info.scriptPath() + '/icons/folder.png'), '', this.dlg);
  this.btOpenCacheTranslations.clicked.connect(this, this.openCacheTranslations);

  this.ckCacheRomaji = new QCheckBox(TEXT_CONFIGURE_DIALOG_CBCACHEROMAJI, this.dlg);
  this.btDeleteCacheRomaji = new QPushButton(new QIcon(Amarok.Info.scriptPath() + '/icons/erase.png'), TEXT_CONFIGURE_DIALOG_BTDELETE_CACHE, this.dlg);
  this.btDeleteCacheRomaji.clicked.connect(this, this.deleteCacheRomaji);
  this.btDeleteCacheRomaji.toolTip = this.calculateCacheSize(PATH_ROMAJI);
  this.btOpenCacheRomaji = new QPushButton(new QIcon(Amarok.Info.scriptPath() + '/icons/folder.png'), '', this.dlg);
  this.btOpenCacheRomaji.clicked.connect(this, this.openCacheRomaji);

  this.ckCacheKana = new QCheckBox(TEXT_CONFIGURE_DIALOG_CBCACHEKANA, this.dlg);
  this.btDeleteCacheKana = new QPushButton(new QIcon(Amarok.Info.scriptPath() + '/icons/erase.png'), TEXT_CONFIGURE_DIALOG_BTDELETE_CACHE, this.dlg);
  this.btDeleteCacheKana.clicked.connect(this, this.deleteCacheKana);
  this.btDeleteCacheKana.toolTip = this.calculateCacheSize(PATH_KANA);
  this.btOpenCacheKana = new QPushButton(new QIcon(Amarok.Info.scriptPath() + '/icons/folder.png'), '', this.dlg);
  this.btOpenCacheKana.clicked.connect(this, this.openCacheKana);

  this.ckCacheRomaja = new QCheckBox(TEXT_CONFIGURE_DIALOG_CBCACHEROMAJA, this.dlg);
  this.btDeleteCacheRomaja = new QPushButton(new QIcon(Amarok.Info.scriptPath() + '/icons/erase.png'), TEXT_CONFIGURE_DIALOG_BTDELETE_CACHE, this.dlg);
  this.btDeleteCacheRomaja.clicked.connect(this, this.deleteCacheRomaja);
  this.btDeleteCacheRomaja.toolTip = this.calculateCacheSize(PATH_ROMAJA);
  this.btOpenCacheRomaja = new QPushButton(new QIcon(Amarok.Info.scriptPath() + '/icons/folder.png'), '', this.dlg);
  this.btOpenCacheRomaja.clicked.connect(this, this.openCacheRomaja);

  this.lbLanguage = new QLabel(TEXT_CONFIGURE_DIALOG_LBLANGUAGE, this.dlg);
  this.cbLanguage = new QComboBox(this.dlg);

  this.lbShowMethod = new QLabel(TEXT_CONFIGURE_DIALOG_LBSHOWMETHOD, this.dlg);
  this.cbShowMethod = new QComboBox(this.dlg);

  this.lbToKanaServer = new QLabel(TEXT_CONFIGURE_DIALOG_LBTOKANASERVER, this.dlg);
  this.cbToKanaServer = new QComboBox(this.dlg);

  this.lbToRomajiServer = new QLabel(TEXT_CONFIGURE_DIALOG_LBTOROMAJISERVER, this.dlg);
  this.cbToRomajiServer = new QComboBox(this.dlg);

  this.glFields.addWidget(this.ckRepeatSearch, 10, 0);
  this.glFields.addWidget(this.ckRepeatSearchLocalOnly, 10, 40);

  this.glFields.addWidget(this.ckCacheLyrics, 20, 0);
  this.glFields.addWidget(this.btDeleteCacheLyrics, 20, 40);
  this.glFields.addWidget(this.btOpenCacheLyrics, 20, 60);

  this.glFields.addWidget(this.ckCacheTranslations, 30, 0);
  this.glFields.addWidget(this.btDeleteCacheTranslations, 30, 40);
  this.glFields.addWidget(this.btOpenCacheTranslations, 30, 60);

  this.glFields.addWidget(this.ckCacheRomaji, 40, 0);
  this.glFields.addWidget(this.btDeleteCacheRomaji, 40, 40);
  this.glFields.addWidget(this.btOpenCacheRomaji, 40, 60);

  this.glFields.addWidget(this.ckCacheKana, 50, 0);
  this.glFields.addWidget(this.btDeleteCacheKana, 50, 40);
  this.glFields.addWidget(this.btOpenCacheKana, 50, 60);

  this.glFields.addWidget(this.ckCacheRomaja, 60, 0);
  this.glFields.addWidget(this.btDeleteCacheRomaja, 60, 40);
  this.glFields.addWidget(this.btOpenCacheRomaja, 60, 60);

  this.glFields.addWidget(this.lbLanguage, 70, 0);
  this.glFields.addWidget(this.lbToKanaServer, 70, 40);
  this.glFields.addWidget(this.cbLanguage, 80, 0);
  this.glFields.addWidget(this.cbToKanaServer, 80, 40);

  this.glFields.addWidget(this.lbShowMethod, 90, 0);
  this.glFields.addWidget(this.lbToRomajiServer, 90, 40);
  this.glFields.addWidget(this.cbShowMethod, 100, 0);
  this.glFields.addWidget(this.cbToRomajiServer, 100, 40);

  this.grFields.setLayout(this.glFields);

  // Buttons group.
  this.hblButtons = new QHBoxLayout();
  this.hblButtons.addStretch();
  this.btOk = new QPushButton(new QIcon(Amarok.Info.scriptPath() + '/icons/ok.png'), TEXT_CONFIGURE_DIALOG_BTOK, this.dlg);
  this.btOk.clicked.connect(this, this.accept);
  this.btOk.clicked.connect(this, this.close);
  this.btCancel = new QPushButton(new QIcon(Amarok.Info.scriptPath() + '/icons/cancel.png'), TEXT_CONFIGURE_DIALOG_BTCANCEL, this.dlg);
  this.btCancel.clicked.connect(this, this.close);
  this.hblButtons.addWidget(this.btOk, 0, 0);
  this.hblButtons.addWidget(this.btCancel, 0, 0);

  // Main layout.
  this.vblMain = new QVBoxLayout(this.dlg);
  this.vblMain.addWidget(this.grFields, 0, 0);
  this.vblMain.addStretch();
  this.vblMain.addLayout(this.hblButtons);

  this.dlg.setLayout(this.vblMain);
  this.dlg.sizeHint = new QSize(530, 430);
  this.dlg.size = new QSize(530, 430);

  this.dlg.setWindowTitle(TEXT_SCRIPT_NAME + ' - ' + TEXT_CONFIGURE_DIALOG_TITLE);

  // Añadimos los datos.
  this.ckRepeatSearch.checked = cfg[CFG_REPEAT_SEARCH];
  this.ckRepeatSearch.stateChanged.connect(this, this.ckRepeatSearch_stateChanged);
  this.ckRepeatSearchLocalOnly.checked = cfg[CFG_REPEAT_SEARCH_INCOLLECTION_ONLY];
  this.ckRepeatSearchLocalOnly.enabled = this.ckRepeatSearch.checked;
  this.ckCacheKana.checked = cfg[CFG_CACHE_KANA];
  //this.ckCacheKorean.checked = cfg[CFG_CACHE_KOREAN];
  this.ckCacheLyrics.checked = cfg[CFG_CACHE_LYRICS];
  this.ckCacheRomaja.checked = cfg[CFG_CACHE_ROMAJA];
  this.ckCacheRomaji.checked = cfg[CFG_CACHE_ROMAJI];
  this.ckCacheTranslations.checked = cfg[CFG_CACHE_TRANSLATION];

  this.languages = new Array('ar', 'bg', 'ca', 'cs', 'da', 'de', 'el', 'en', 'es', 'et', 'fi', 'fr', 'gl', 'hi', 'hr', 'hu', 'id', 'it', 'iw', 'ja', 'ko', 'lt', 'lv', 'mt', 'nl', 'no', 'pl', 'pt', 'ro', 'ru', 'sk', 'sl', 'sq', 'sr', 'sv', 'th', 'tl', 'tr', 'uk', 'vi', 'zh-CN', 'zh-TW');
  this.cbLanguage.addItems(this.languages);
  // Alucino mucho, los array no tienen búsquedas.
  // ¡¡¡ MENUDA MIERDA DE LENGUAGE !!!
  for (i = 0; i < this.languages.length; i++) {

    if (cfg[CFG_TRANSLATION_LANGUAGE] == this.languages[i]) {
      this.cbLanguage.currentIndex = i;
      break;
    }

  } // for (i = 0; i < languages.length; i++)

  this.showMethods = new Array('xml', 'html', 'xml, html', 'html, xml')
  this.cbShowMethod.addItems(this.showMethods);
  this.cbShowMethod.currentIndex = cfg[CFG_SHOW_METHOD];

  this.kanjiConversionServers = new Array('tatoeba.org', 'yomoyomo.jp');
  this.cbToKanaServer.addItems(this.kanjiConversionServers);
  this.cbToKanaServer.currentIndex = cfg[CFG_CONVERSION_KANA_SERVER];

  this.kanjiConversionServers = new Array('tatoeba.org', 'yomoyomo.jp');
  this.cbToRomajiServer.addItems(this.kanjiConversionServers);
  this.cbToRomajiServer.currentIndex = cfg[CFG_CONVERSION_ROMAJI_SERVER];

  return true;

} // function configurationEditor()

function readConfiguration() {

  /*
  cfg[CFG_FILE_NAME] = 'LWS.cfg';
  cfg[CFG_CACHE_KANA] = true;
  cfg[CFG_CACHE_KOREAN] = true;
  cfg[CFG_CACHE_LYRICS] = true;
  cfg[CFG_CACHE_ROMAJA] = true;
  cfg[CFG_CACHE_ROMAJI] = true;
  cfg[CFG_CACHE_TRANSLATION] = true;
  cfg[CFG_EXTERNAL_LYRICS_SHOW] = false;
  cfg[CFG_TRANSLATION_LANGUAGE] = QLocale.system().name().substring(0, 2);;
  cfg[CFG_REPEAT_SEARCH] = true;
  cfg[CFG_REPEAT_SEARCH_INCOLLECTION_ONLY] = true;
  cfg[CFG_SHOW_METHOD] = 2;
  cfg[CFG_CONVERSION_KANA_SERVER] = 1;
  cfg[CFG_CONVERSION_ROMAJI_SERVER] = 1;
  */

  try {

    var file = new QFile(Amarok.Info.scriptPath() + '/' + cfg[CFG_FILE_NAME]);
    if (!file.exists()) { return true; }
    file.open(new QIODevice.OpenMode(QIODevice.ReadWrite));
    var data = new QTextStream(file);

    var line = '';
    var propiedad = '';
    var valor = '';

    while (!data.atEnd()) {

      line = trim(data.readLine());

      if ((line == '') || (line[0] == '#')) { continue; };

      line = line.split('=');
      if (line[1] == undefined) { continue; };
      if (line[2] != undefined) { continue; };

      propiedad = trim(line[0].toLowerCase());
      valor = trim(line[1]);

      switch(propiedad) {

        case 'cache':
          cfg[CFG_CACHE_KANA] = ((valor.substring(0, 1) == '1') ? true : false);
          cfg[CFG_CACHE_KOREAN] = true; //((valor.substring(4, 5) == '1') ? true : false);
          cfg[CFG_CACHE_LYRICS] = ((valor.substring(1, 2) == '1') ? true : false);
          cfg[CFG_CACHE_ROMAJA] = ((valor.substring(2, 3) == '1') ? true : false);
          cfg[CFG_CACHE_ROMAJI] = ((valor.substring(3, 4) == '1') ? true : false);
          cfg[CFG_CACHE_TRANSLATION] = ((valor.substring(4, 5) == '1') ? true : false);
          break

        case 'externalshow':
          cfg[CFG_EXTERNAL_LYRICS_SHOW] = ((valor == '1') ? true : false);
          break

        case 'repeatsearch':
          cfg[CFG_REPEAT_SEARCH] = ((valor == '1') ? true : false);
          break

        case 'repeatsearchincollectiononly':
          cfg[CFG_REPEAT_SEARCH_INCOLLECTION_ONLY] = ((valor == '1') ? true : false);
          break

        case 'showmethod':
          cfg[CFG_SHOW_METHOD] = parseInt(valor, 10);
          break

        case 'translate':
          cfg[CFG_TRANSLATION_LANGUAGE] = valor;
          break

        case 'tokanaserver':
          cfg[CFG_CONVERSION_KANA_SERVER] = parseInt(valor, 10);
          break

        case 'toromajiserver':
          cfg[CFG_CONVERSION_ROMAJI_SERVER] = parseInt(valor, 10);
          break

        default:

      } // switch(variable)

    } // while (!data.atEnd())

    file.close();

    equivalences.sort();

  } catch(err) {
        Amarok.debug('error: ' + err);
  }

} // function readConfiguration()


function configureDialog() {

  var ce = new configurationEditor();
  ce.show();

  return true;

} // function configureDialog()
