/**************************************************************************
*   Amarok 2 script to manage lyrics                                      *
*                                                                         *
*   Copyright                                                             *
*   (C) 2008 Ignacio Serantes <kde@aynoa.net>                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.         *
**************************************************************************/


//
// Conversión de Kanji a kana
//

function onDownloadKana(data) {

  if (data != '') {

    var fileName = getNameExternalFileTrack();
    var te = new textEditor(data, fileName, 'K', '"' + Amarok.Engine.currentTrack().title + '" ' + TEXT_HTML_BY + ' "' + Amarok.Engine.currentTrack().artist + '"');
    te.btnReload.clicked.connect(toKanaAgain);
    te.show();

    if (cfg[CFG_CACHE_KANA]) {
      saveTextFile(Amarok.Info.scriptPath() + '/' + PATH_KANA, fileName, data);
    }

    Amarok.Window.Statusbar.shortMessage(TEXT_CONVERSION_KANA_SUCCESS);

  } else {

    Amarok.Window.Statusbar.shortMessage(TEXT_CONVERSION_KANA_FAILS);

  } // if (data != '')

} // function onDownloadKana(data)

function toKana() {

  // Si amarok no está tocando no seguimos.
  if (Amarok.Engine.engineState() == AMAROK_ENGINE_STATE_STOP) {
    Amarok.Window.Statusbar.longMessage(TEXT_MSG_OPTION_DISABLED);
    return true;
  }

  var fileName = getNameExternalFileTrack();
  var convertedData = readTextFile(Amarok.Info.scriptPath() + '/' + PATH_KANA + '/' + fileName);
  if (convertedData != '') {

    var te = new textEditor(convertedData, fileName, 'K', '"' + Amarok.Engine.currentTrack().title + '" ' + TEXT_HTML_BY + ' "' + Amarok.Engine.currentTrack().artist + '"');
    te.btnReload.clicked.connect(toKanaAgain);
    te.show();

  } else {

    var data = readStoredLyrics();
    if (data == '') { return true; }

    Amarok.Window.Statusbar.shortMessage(TEXT_CONVERSION_KANA_START);

    switch(cfg[CFG_CONVERSION_KANA_SERVER]) {

      case CONVERSION_SERVER_TATOEBA:
        new kanjiConversion(data, CONVERSION_TATOEBA_KANA, onDownloadKana);
        break;

      case CONVERSION_SERVER_YOMOYOMO:
        new kanjiConversion(data, CONVERSION_YOMOYOMO_KANA, onDownloadKana);
        break;

      default:

    } // switch(cfg[CFG_CONVERSION_KANA_SERVER]) {

  } // if (convertedData != '')

  return true;

} // function toKana()

function toKanaAgain() {

  // Si amarok no está tocando no seguimos.
  if (Amarok.Engine.engineState() == AMAROK_ENGINE_STATE_STOP) {
    Amarok.Window.Statusbar.longMessage(TEXT_MSG_OPTION_DISABLED);
    return true;
  }

  var data = readStoredLyrics();
  if (data == '') { return true; }

  Amarok.Window.Statusbar.shortMessage(TEXT_CONVERSION_KANA_START);

  switch(cfg[CFG_CONVERSION_KANA_SERVER]) {

    case CONVERSION_SERVER_TATOEBA:
      new kanjiConversion(data, CONVERSION_TATOEBA_KANA, onDownloadKana);
      break;

    case CONVERSION_SERVER_YOMOYOMO:
      new kanjiConversion(data, CONVERSION_YOMOYOMO_KANA, onDownloadKana);
      break;

    default:

  } // switch(cfg[CFG_CONVERSION_KANA_SERVER]) {


} // function toKanaAgain()


//
// Conversión de Kanji a rōmaji
//

function onDownloadRomaji(data) {

  if (data != '') {

    var fileName = getNameExternalFileTrack();
    var te = new textEditor(data, fileName, 'R', '"' + Amarok.Engine.currentTrack().title + '" ' + TEXT_HTML_BY + ' "' + Amarok.Engine.currentTrack().artist + '"');
    te.btnReload.clicked.connect(toRomajiAgain);
    te.show();

    if (cfg[CFG_CACHE_ROMAJI]) {
      saveTextFile(Amarok.Info.scriptPath() + '/' + PATH_ROMAJI, fileName, data);
    }

    Amarok.Window.Statusbar.shortMessage(TEXT_CONVERSION_ROMAJI_SUCCESS);

  } else {

    Amarok.Window.Statusbar.shortMessage(TEXT_CONVERSION_ROMAJI_FAILS);

  } // if (data != '')

} // function onDownloadRomaji(data)

function toRomaji() {

  // Si amarok no está tocando no seguimos.
  if (Amarok.Engine.engineState() == AMAROK_ENGINE_STATE_STOP) {
    Amarok.Window.Statusbar.longMessage(TEXT_MSG_OPTION_DISABLED);
    return true;
  }

  var fileName = getNameExternalFileTrack();
  var convertedData = readTextFile(Amarok.Info.scriptPath() + '/' + PATH_ROMAJI + '/' + fileName);
  if (convertedData != '') {

    var te = new textEditor(convertedData, fileName, 'R', '"' + Amarok.Engine.currentTrack().title + '" ' + TEXT_HTML_BY + ' "' + Amarok.Engine.currentTrack().artist + '"');
    te.btnReload.clicked.connect(toRomajiAgain);
    te.show();

  } else {

    var data = readStoredLyrics();
    if (data == '') { return true; }

    Amarok.Window.Statusbar.shortMessage(TEXT_CONVERSION_ROMAJI_START);

    switch(cfg[CFG_CONVERSION_ROMAJI_SERVER]) {

      case CONVERSION_SERVER_TATOEBA:
        new kanjiConversion(data, CONVERSION_TATOEBA_ROMAJI, onDownloadRomaji);
        break;

      case CONVERSION_SERVER_YOMOYOMO:
        new kanjiConversion(data, CONVERSION_YOMOYOMO_ROMAJI, onDownloadRomaji);
        break;

      default:

    } // switch(cfg[CFG_CONVERSION_ROMAJI_SERVER])

  } // if (convertedData != '')

  return true;

} // function toRomaji()

function toRomajiAgain() {

  // Si amarok no está tocando no seguimos.
  if (Amarok.Engine.engineState() == AMAROK_ENGINE_STATE_STOP) {
    Amarok.Window.Statusbar.longMessage(TEXT_MSG_OPTION_DISABLED);
    return true;
  }

  var data = readStoredLyrics();
  if (data == '') { return true; }

  Amarok.Window.Statusbar.shortMessage(TEXT_CONVERSION_ROMAJI_START);

  switch(cfg[CFG_CONVERSION_ROMAJI_SERVER]) {

    case CONVERSION_SERVER_TATOEBA:
      new kanjiConversion(data, CONVERSION_TATOEBA_ROMAJI, onDownloadRomaji);
      break;

    case CONVERSION_SERVER_YOMOYOMO:
      new kanjiConversion(data, CONVERSION_YOMOYOMO_ROMAJI, onDownloadRomaji);
      break;

    default:

  } // switch(cfg[CFG_CONVERSION_ROMAJI_SERVER])

} // function toRomajiAgain()


//
// Conversión de Kanji a koreano
//

function onDownloadKorean(data) {

  if (data != '') {

    var fileName = getNameExternalFileTrack();
    var te = new textEditor(data, fileName, 'O', '"' + Amarok.Engine.currentTrack().title + '" ' + TEXT_HTML_BY + ' "' + Amarok.Engine.currentTrack().artist + '"');
    te.btnReload.clicked.connect(toKoreanAgain);
    te.show();

    if (cfg[CFG_CACHE_KOREAN]) {
      saveTextFile(Amarok.Info.scriptPath() + '/' + PATH_KOREAN, fileName, data);
    }

    Amarok.Window.Statusbar.shortMessage(TEXT_CONVERSION_KOREAN_SUCCESS);

  } else {

    Amarok.Window.Statusbar.shortMessage(TEXT_CONVERSION_KOREAN_FAILS);

  } // if (data != '')

} // function onDownloadKorean(data)

function toKorean() {

  // Si amarok no está tocando no seguimos.
  if (Amarok.Engine.engineState() == AMAROK_ENGINE_STATE_STOP) {
    Amarok.Window.Statusbar.longMessage(TEXT_MSG_OPTION_DISABLED);
    return true;
  }

  var fileName = getNameExternalFileTrack();
  var convertedData = readTextFile(Amarok.Info.scriptPath() + '/' + PATH_KOREAN + '/' + fileName);
  if (convertedData != '') {

    var te = new textEditor(convertedData, fileName, 'O', '"' + Amarok.Engine.currentTrack().title + '" ' + TEXT_HTML_BY + ' "' + Amarok.Engine.currentTrack().artist + '"');
    te.btnReload.clicked.connect(toKoreanAgain);
    te.show();

  } else {

    var data = readStoredLyrics();
    if (data == '') { return true; }

    Amarok.Window.Statusbar.shortMessage(TEXT_CONVERSION_KOREAN_START);

    new kanjiConversion(data, CONVERSION_YOMOYOMO_KOREAN, onDownloadKorean);

  } // if (convertedData != '')

  return true;

} // function toKorean()

function toKoreanAgain() {

  // Si amarok no está tocando no seguimos.
  if (Amarok.Engine.engineState() == AMAROK_ENGINE_STATE_STOP) {
    Amarok.Window.Statusbar.longMessage(TEXT_MSG_OPTION_DISABLED);
    return true;
  }

  var data = readStoredLyrics();
  if (data == '') { return true; }

  Amarok.Window.Statusbar.shortMessage(TEXT_CONVERSION_KOREAN_START);
  new kanjiConversion(data, CONVERSION_YOMOYOMO_KOREAN, onDownloadKorean);

} // function toKoreanAgain()


//
// Romanización del Hangul.
//
function toRomaja() {

  // Si amarok no está tocando no seguimos.
  if (Amarok.Engine.engineState() == AMAROK_ENGINE_STATE_STOP) {
    Amarok.Window.Statusbar.longMessage(TEXT_MSG_OPTION_DISABLED);
    return true;
  }

  var fileName = getNameExternalFileTrack();
  var convertedData = readTextFile(Amarok.Info.scriptPath() + '/' + PATH_ROMAJA + '/' + fileName);

  if (convertedData != '') {

    var te = new textEditor(convertedData, fileName, 'H', '"' + Amarok.Engine.currentTrack().title + '" ' + TEXT_HTML_BY + ' "' + Amarok.Engine.currentTrack().artist + '"');
    te.btnReload.clicked.connect(toRomajaAgain);
    te.show();

  } else {

    toRomajaAgain();

  } // if (convertedData != '')

  return true;

} // function toRomaja()

function toRomajaAgain() {

  // Si amarok no está tocando no seguimos.
  if (Amarok.Engine.engineState() == AMAROK_ENGINE_STATE_STOP) {
    Amarok.Window.Statusbar.longMessage(TEXT_MSG_OPTION_DISABLED);
    return true;
  }

  var data = readStoredLyrics();
  if (data == '') { return true; }

  Amarok.Window.Statusbar.shortMessage(TEXT_CONVERSION_ROMAJA_START);

  var convertedData = fromHangulToRomaja(data);
  var fileName = getNameExternalFileTrack();
  var te = new textEditor(convertedData, fileName, 'H', '"' + Amarok.Engine.currentTrack().title + '" ' + TEXT_HTML_BY + ' "' + Amarok.Engine.currentTrack().artist + '"');
    te.btnReload.clicked.connect(toRomajaAgain);
    te.show();
  if (cfg[CFG_CACHE_ROMAJA]) {
    saveTextFile(Amarok.Info.scriptPath() + '/' + PATH_ROMAJA, fileName, convertedData);
  }

  Amarok.Window.Statusbar.shortMessage(TEXT_CONVERSION_ROMAJA_SUCCESS);

} // function toRomaja()


//
// Traducción.
//

function onDownloadTranslate() {

  var data = Amarok.Lyrics.toUtf8(http.readAll());

  // Puede tener su utilidad en el futuro.
  if (data.substring(data.length-2, data.length) == ']]') {

    // Formato 2 ["texto", "lang", ["significados"]]
    var length = data.indexOf('",[[');
    var langOrig = data.substring(length - 3, length);
    data = data.substring(2, length - 5).replace(/\\n /g, '\n').replace(/\\n/g,'\n');

  } else {

    // Formato 1 ["texto","lang"] lang=es, en, ko, ja, etc...
    var length = data.length;
    var langOrig = data.substring(length - 4, length - 2);
    data = data.substring(2, length - 7).replace(/\\n /g, '\n').replace(/\\n/g,'\n');

  } // if (data.substring(data.length-2, data.length) == ']]')

  var fileName = getNameExternalFileTrack();
  var te = new textEditor (data, fileName, 'T', '"' + Amarok.Engine.currentTrack().title + '" ' + TEXT_HTML_BY + ' "' + Amarok.Engine.currentTrack().artist + '"');
  te.btnReload.clicked.connect(translateAgain);
  te.show();

  if (cfg[CFG_CACHE_TRANSLATION]) {
    saveTextFile(Amarok.Info.scriptPath() + '/' + PATH_TRANSLATIONS, fileName, data);
  }

  Amarok.Window.Statusbar.shortMessage(TEXT_TRANSLATION_SUCCESS);

  return true;

} // function onDownloadTranslate()

function translate(again) {

  // Si amarok no está tocando no seguimos.
  if (Amarok.Engine.engineState() == AMAROK_ENGINE_STATE_STOP) {
    Amarok.Window.Statusbar.longMessage(TEXT_MSG_OPTION_DISABLED);
    return true;
  }

  if (again == null) {

    var fileName = getNameExternalFileTrack();
    var readData = readTextFile(Amarok.Info.scriptPath() + '/' + PATH_TRANSLATIONS + '/' + fileName);
    if (readData != '') {

      var te = new textEditor (readData, fileName, 'T', '"' + Amarok.Engine.currentTrack().title + '" ' + TEXT_HTML_BY + ' "' + Amarok.Engine.currentTrack().artist + '"');
      te.btnReload.clicked.connect(translateAgain);
      te.show();

      return true;

    } // if (readData != '')

  } // if (again == null)

  var data = readStoredLyrics();

  if (data != '') {

    Amarok.Window.Statusbar.shortMessage(TEXT_TRANSLATION_START);

    // Procesamos los carácteres que dan problemas con la traducción.
    data = data.replace(/&/g, 'and');

    try {

      var langOrig = 'auto';
      var langDest = cfg[CFG_TRANSLATION_LANGUAGE];

      http = new QHttp();
      http.done.connect(onDownloadTranslate);
      http.setHost('www.google.com');
      http.post('/translate_a/t?client=t&sl=' + langOrig + '&tl=' + langDest, Amarok.Lyrics.fromUtf8('hl=en&oe=UTF-8&ie=UTF-8&text=' + data, 'UTF-8'));

    } catch(err) {
          Amarok.debug('error: ' + err);
    }

  } // if (data != '')

  return true;

} // function translate()


function translateAgain() {

  translate('');

} // function translateAgain()


//
// Clase kanjiConversion
//

function kanjiConversion(data, conversion, func) {

  this.toRomaji = function(data) {

    var convertedData = '';

    data = getDataBetweenTags(data, '<div id="conversion">', '</div>', 1);

    if ((data != '') && (data != '<br />\n')) {

      var valido = true;
      for (var i = 0; i < data.length; i++) {

        aChar = data[i];
        charCode = aChar.charCodeAt(0);

        // Los hiragana están entre 12353 small "ぁ" y 12438 small "ゖ".
        // Los katakana estén entre 12448 "゠" y 12543 "ヿ".

        if (valido) {

          if (aChar == ']') {
            convertedData += aChar;
            continue;
          } //if (aChar == ']')

          // Esto va un poco a ciegas pero parece que funciona.
          if (charCode <= 12543) {
            convertedData += aChar;
            continue;
          } else {
            valido = false;
          }
        } // if (valido)

        if ((aChar == '[') || (aChar == ']')) {
          valido = true;
        }

      } // for (var i = 0; i < data.length; i++)

      // Kakasi parece gestionar mal el carácter "'".
      convertedData = convertedData.replace(/¥'/g, "'");

    } //if ((data != '') && (data != '<br />\n'))

    return convertedData;

  } // this.toRomaji = function(data)

  this.toKana = function(data) {

    var convertedData = '';

    data = getDataBetweenTags(data, '<div id="conversion">', '</div>', 1);

    if ((data != '') && (data != '<br />\n')) {

      var valido = true;
      for (var i = 0; i < data.length; i++) {

        aChar = data[i];
        charCode = aChar.charCodeAt(0);

        // Los hiragana están entre 12353 small "ぁ" y 12438 small "ゖ".
        // Los katakana estén entre 12448 "゠" y 12543 "ヿ".

        if (valido) {

          if (aChar == ']') {  continue; }

          // Esto va un poco a ciegas pero parece que funciona.
          if (charCode <= 12543) {
            convertedData += aChar;
            continue;
          } else {
            valido = false;
          }
        } // if (valido)

        if ((aChar == '[') || (aChar == ']')) {
          valido = true;
        }

      } // for (var i = 0; i < data.length; i++)

      // Kakasi parece gestionar mal el carácter "'".
      convertedData = convertedData.replace(/¥'/g, "'");

    } //if ((data != '') && (data != '<br />\n'))

    return convertedData;

  } // this.toKana = function(data)

  this.cleanData = function(data) {

    data = getDataBetweenTags(data, '<td align="left">', '</td>', 1)
    data = data.replace(/<rb>[^<\/rb>]*<\/rb>/g, '');
    data = data.replace(/<rp>[^<\/rp>]*<\/rp>/g, '');
    data = data.replace(/<[^>]*>/g, '');
    return html2text(data);

  } // this.cleanData()

  this.onFinished = function(networkReply) {

    var data = Amarok.Lyrics.toUtf8(networkReply.readAll());

    if (this.conversionType != '') {
      if (this.conversionType == 'romaji') {
        this.func(this.toRomaji(data));
      } else {
        this.func(this.toKana(data));
      }
      return true
    } // if (this.conversionType != '')

    if (this.yyparam != '') {
      this.func(this.cleanData(data));
      return true
    } // if (this.yyparam != '')

    return true;

  } // this.onFinished = function()

  //this.data = data.replace(/\\n/g, '<br />');
  this.data = data;
  this.conversion = conversion;
  this.func = func;

  if (data != '') {

    switch(this.conversion) {

      case CONVERSION_TATOEBA_KANA:
        this.serverUrl = 'http://tatoeba.org/eng/tools/kakasi';
        this.conversionType = 'furigana';
        this.yyparam = '';
        break;

      case CONVERSION_TATOEBA_ROMAJI:
        this.serverUrl = 'http://tatoeba.org/eng/tools/kakasi';
        this.conversionType = 'romaji';
        this.yyparam = '';
        break;

      case CONVERSION_YOMOYOMO_KANA:
        this.serverUrl = 'http://yomoyomo.jp/text.php';
        this.conversionType = '';
        this.yyparam = '00000111';
        break;

      case CONVERSION_YOMOYOMO_ROMAJI:
        this.serverUrl = 'http://yomoyomo.jp/text.php';
        this.conversionType = '';
        this.yyparam = '00200111';
        break;

      case CONVERSION_YOMOYOMO_KOREAN:
        this.serverUrl = 'http://yomoyomo.jp/text.php';
        this.conversionType = '';
        this.yyparam = '00300111';
        break;

      case CONVERSION_YOMOYOMO_ARABIC:
        this.serverUrl = 'http://yomoyomo.jp/text.php';
        this.conversionType = '';
        this.yyparam = '00400111';
        break;

      case CONVERSION_YOMOYOMO_RUSSIAN:
        this.serverUrl = 'http://yomoyomo.jp/text.php';
        this.conversionType = '';
        this.yyparam = '00500111';
        break;

      default:
        return false;

    } // switch(conversion) {

    this.nam = new QNetworkAccessManager();
    this.nam.finished.connect(this, this.onFinished);
    this.url = new QUrl(this.serverUrl);

    if (this.conversionType != '') {

      this.url.addQueryItem('query', this.data);
      this.url.addQueryItem('type', this.conversionType);
      this.request = new QNetworkRequest(this.url);
      this.nam.get(this.request);
      return true;

    } // if (this.conversionType != '')

    if (this.yyparam != '') {
      this.url.addQueryItem('yyparam', this.yyparam);
      this.url.addQueryItem('t', '');
      this.request = new QNetworkRequest(this.url);
      this.nam.post(this.request, Amarok.Lyrics.fromUtf8('inputtext=' + this.data, 'UTF-8'));
      return true

    } // if (this.yyparam != '')

  } // if (data != '')

  return true;

} // function kanjiConversion(data, conversion, func)

const CONVERSION_TATOEBA_KANA = 1;
const CONVERSION_TATOEBA_ROMAJI = 2;
const CONVERSION_YOMOYOMO_ARABIC = 3;
const CONVERSION_YOMOYOMO_KANA = 4;
const CONVERSION_YOMOYOMO_KOREAN = 5;
const CONVERSION_YOMOYOMO_ROMAJI = 6;
const CONVERSION_YOMOYOMO_RUSSIAN = 7;

const CONVERSION_SERVER_TATOEBA = 0;
const CONVERSION_SERVER_YOMOYOMO = 1;

var http = null;
