/**************************************************************************
*   Amarok 2 lyrics to manage lyrics                                      *
*                                                                         *
*   Copyright                                                             *
*   (C) 2008 Ignacio Serantes <kde@aynoa.net>                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.         *
**************************************************************************/

/*
  TODO: Pantalla de búsqueda.
    [X] Escritura de la pantalla
    [X] Cierre automático
    [X] Soporte de canciones instrumentales
    [X] Capacidad de ignorar la cache
  TODO: Pantalla para visualizar textos.
    [X] funciones básicas: editar y grabar.
    [ ] borrado de cache.
  TODO: conversión a kana y romaji.
    [X] usando GET y tatoeba.org
    [X] nuevo motor con soporte multiservidor
    [X] usar POST y yomoyomo.jp y pasar del muy limitado Downloader()
    [ ] añadir todas las posibilidades (árabe, ruso y koreano) de yomomoyomo
  TODO: conversión a romaja.
    [X] función de conversión
  TODO:
    [X] traducción usando google translation.
    [X] usar método POST y pasar de Downloader()
  TODO: cache.
    [X] kana
    [X] romaji
    [X] romaja
    [X] traducciones
    [X] letras
  TODO: pantalla de configuración:
    [X] repetición de búsqueda
    [X] habilitar cache
    [X] borrado de la cache
    [X] grabar y restaurar configuración
    [X] informar del número de ficheros y del espacio consumido en la cache
    [X] selección de idioma para la traducción
    [X] selección del modo de almacenamiento de las letras (XML y/o HTML)
    [X] selección del servidor para la conversión de kanji a kana y rōmaji
    [ ] configuración de motores de búsqueda.
  TODO: reescribir los motores de búsqueda como plugins.
    [ ] cada motor un plugin en un fichero individual.
    [ ] registro automático de motores
  TODO: gestión de equivalencias
    [X] gestión básica (edición manual del fichero de texto)
    [ ] gestión avanzada: alta, baja, modificación.
    [ ] alta automática al acertar la búsqueda.
  TODO: iconos
    [X] añadir iconos a todos los botones y donde haga falta
    [ ] poner el icono de "Lyrics Workshop" a la pantalla de configuración
  TODO: multilenguaje
    [X] english
    [ ] galego
    [ ] japonés
    [ ] otros

  TODO: soporte de otros idiomas: ruso, chino, etc..., si hay servidores
  TODO: visualizador de letras usando plasma. ¿STDIN?, ¿otro?, ¿uno propio?.
  TODO: karaoke
    [ ] soporte del formato lrc básico y avanzado
    [ ] pantalla de karaoke ¿usar plasma, una ventana o los dos?

*/

Importer.loadQtBinding("qt.core");
Importer.loadQtBinding("qt.gui");
Importer.loadQtBinding("qt.network");
//Importer.loadQtBinding("qt.opengl");
//Importer.loadQtBinding("qt.sql");
//Importer.loadQtBinding("qt.svg");
//Importer.loadQtBinding("qt.uitools");
//Importer.loadQtBinding("qt.webkit");
//Importer.loadQtBinding("qt.xml");


// Array cfg.
const CFG_FILE_NAME = 0;
const CFG_CACHE_KANA = 1;
const CFG_CACHE_LYRICS = 2;
const CFG_CACHE_ROMAJA = 3;
const CFG_CACHE_ROMAJI = 4;
const CFG_CACHE_TRANSLATION = 5;
const CFG_TRANSLATION_LANGUAGE = 6;
const CFG_REPEAT_SEARCH = 7;
const CFG_EXTERNAL_LYRICS_SHOW = 8;
const CFG_SHOW_METHOD = 9;
const CFG_CONVERSION_KANA_SERVER = 10;
const CFG_CONVERSION_ROMAJI_SERVER = 11;
const CFG_REPEAT_SEARCH_INCOLLECTION_ONLY = 12;
const CFG_CACHE_KOREAN = 13;

var cfg = new Array();
cfg[CFG_FILE_NAME] = 'LWS.cfg';
cfg[CFG_CACHE_KANA] = true;
cfg[CFG_CACHE_LYRICS] = true;
cfg[CFG_CACHE_ROMAJA] = true;
cfg[CFG_CACHE_ROMAJI] = true;
cfg[CFG_CACHE_TRANSLATION] = true;
cfg[CFG_EXTERNAL_LYRICS_SHOW] = false;
cfg[CFG_TRANSLATION_LANGUAGE] = QLocale.system().name().substring(0, 2);;
cfg[CFG_REPEAT_SEARCH] = true;
cfg[CFG_REPEAT_SEARCH_INCOLLECTION_ONLY] = true;
cfg[CFG_SHOW_METHOD] = 2;
cfg[CFG_CONVERSION_KANA_SERVER] = 1;
cfg[CFG_CONVERSION_ROMAJI_SERVER] = 0;
cfg[CFG_CACHE_KOREAN] = true;

Importer.include('functions.js');
Importer.include('cfg.js');
// Amarok.Config.readConfig() no funciona.
readConfiguration();


// Constantes de amarok.
//const AMAROK_ENGINE_STATE_PLAY = 0;
//const AMAROK_ENGINE_STATE_PAUSE = 1;
const AMAROK_ENGINE_STATE_STOP = 2;

// Directorios.
const PATH_CACHE = 'cache';
const PATH_LYRICS = PATH_CACHE + '/lyrics';
const PATH_KANA = PATH_CACHE + '/kana';
const PATH_KOREAN = PATH_CACHE + '/korean';
const PATH_ROMAJA = PATH_CACHE + '/romaja';
const PATH_ROMAJI = PATH_CACHE + '/romaji';
const PATH_TRANSLATIONS = PATH_CACHE + '/translations';

var languageFileName = 'lang/' + QLocale.system().name().substring(0, 2) + '.js';
if (QFile.exists(Amarok.Info.scriptPath() + '/' + languageFileName)) {
  Importer.include(languageFileName);
} else {
  Importer.include('lang/en.js');
}

Importer.include('db.js');
Importer.include('searchdialog.js');
Importer.include('textdialog.js');
Importer.include('language.js');

Importer.include('engines/cix.js');
Importer.include('engines/evesta.js');
Importer.include('engines/goo.js');
Importer.include('engines/lyricwiki.js');
Importer.include('engines/sogou.js');


//
// Funciones comunes.
//

// Grabamos la letra y la visualizamos.
function saveLyrics(html, ignoreCache) {

  try {

    var ignoreCache = (ignoreCache == null) ? false : ignoreCache;

    // Una comprobación final por si se llega aquí después de un cambio de canción.
    if (initial_artist != '') {
      if ((initial_artist != Amarok.Engine.currentTrack().artist) || (initial_title != Amarok.Engine.currentTrack().title)) {
        return true;
      }
    } // if (initial_artist != '')

    if (!ignoreCache && cfg[CFG_CACHE_LYRICS]) {
      saveTextFile(Amarok.Info.scriptPath() + '/' + PATH_LYRICS, getNameExternalFileTrack('html'), html);
    }

    // Primero limpiamos y luego mostramos.
    // De esta manera parece que la cosa funciona como quiero.
    Amarok.Engine.currentTrack().lyrics = '';
    Amarok.Lyrics.showLyricsHtml(html);
    /*
    switch(cfg[CFG_SHOW_METHOD]) {

      case 0:
        Amarok.Lyrics.showLyrics(html);
        break;

      case 1:
        Amarok.Lyrics.showLyricsHtml(html);
        break;

      case 2:
        Amarok.Lyrics.showLyrics(html);
        Amarok.Lyrics.showLyricsHtml(html);
        break;

      case 3:
        Amarok.Lyrics.showLyricsHtml(html);
        Amarok.Lyrics.showLyrics(html);
        break;

      default:
        Amarok.Lyrics.showLyrics(html);
        Amarok.Lyrics.showLyricsHtml(html);

    } // switch(cfg[CFG_SHOW_METHOD])
    */
    if (cfg[CFG_EXTERNAL_LYRICS_SHOW]) {saveToSTDIN(html)};

  } catch(err) {
        Amarok.debug( "error: " + err );
  }

} // function saveLyrics(html)

// Formateamos para Amarok.
function formatHTML(artist, title, urlArtist, urlLyrics, server, urlServer, lyrics) {

  try {

    var pageTitle = title + " " + artist + " lyrics";
    var footerFrom = (server == '') ? '' : '<strong>' + TEXT_HTML_FETCHED_FROM + '</strong> <a href="' + urlServer + '">' + server + '</a>\n';
    var footerLyrics = (urlLyrics == '') ? '' : '<strong>' + TEXT_HTML_LYRICS + '</strong> <a href="' + urlLyrics + '">' + urlLyrics + '</a>\n';
    var footerArtist = (urlArtist == '') ? '' : '<strong>' + TEXT_HTML_ARTIST + '</strong> <a href="' + urlArtist + '">' + urlArtist + '</a>\n';

    // OJO: no hacerlo con html.replace ya que algunos carácteres en la letra, por ejemplo $10
    // tienen significado y la sustitución hace cosas raras.
    var html = '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">\n<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en" dir="ltr">\n<head>\n<meta http-equiv="Content-Type" content="text/html; charset=utf-8" />\n<title>' + pageTitle + '</title>\n</head>\n<body><h3><a href="' + urlLyrics + '">' + title + '</a> ' + TEXT_HTML_BY + ' <a href="' + urlArtist + '">' + artist + '</a></h3>\n<pre>\n' + lyrics + '</pre><hr/><pre><a href="">' + TEXT_HTML_ADDITIONAL_INFO + footerFrom + footerLyrics + footerArtist + '</pre></body>\n</html>';

  } catch(err) {
        Amarok.debug( "error: " + err );
  }

  return html;

} // function formatHTML(artist, title, urlArtist, urlLyrics, server, urlServer, lyrics)

/*

// engineFinished
function engineFinished(engineId, lyricsFound, urlLyrics, urlArtist, data) {

  try {

    // Incrementamos el contador para determinar cuando ha fallado la búsqueda.
    hitsCount += 1;

    if (lyricsFound) {

      if (searchEngines[engineId][DATA_SERVER_PRIORITY] > lastPriority) { return true};

      data = getDataBetweenTags(data, searchEngines[engineId][DATA_HTML_TAG_BEGIN], searchEngines[engineId][DATA_HTML_TAG_END], searchEngines[engineId][DATA_HTML_TAG_OCURRENCE]);

      if (data != '') {

        Amarok.Window.Statusbar.shortMessage(TEXT_MSG_FOUND);
        lastPriority = searchEngines[engineId][DATA_SERVER_PRIORITY];
        saveLyrics(formatHTML(initial_artist, initial_title, searchGlobals[i][GLOBALS_ARTIST], searchGlobals[i][GLOBALS_LYRIC], searchEngines[i][DATA_SERVER_NAME], searchEngines[i][DATA_SERVER_URL], html2text(data)));

        if (searchGlobals[SEARCH_ENGINE_ALTERNATIVES][GLOBALS_ARTIST] != '') {
          Amarok.Window.Statusbar.longMessage('Using alternative "' + searchGlobals[SEARCH_ENGINE_ALTERNATIVES][GLOBALS_ARTIST] + '"');
        }
      } // if (data != '')

      return true;

    } // if (lyricsFound)


    if ((hitsCount == SEARCH_ENGINES) && (lastPriority == LOWEST_PRIORITY)) {

      if (cfg[CFG_REPEAT_SEARCH]) { lyricsSearch(global_title, global_artist); };

    } // if ((hitsCount == SEARCH_ENGINES) && (lastPriority == 99))

  } catch(err) {
        Amarok.debug( "error: " + err );
  }

}

*/

// Al finalizar la descarga.
function onDownloadFinished(data) {
  try {

    if (data == '') { return true; }

    for (var i = 0; i < SEARCH_ENGINES; i++) {

      // Comprobamos si tiene la marca adecuada.
      if (data.indexOf(searchEngines[i][DATA_SERVER_MARK]) >= 0) {

        // Incrementamos el contador para determinar cuando ha fallado la búsqueda.
        hitsCount += 1;

        // Verificamos que sean datos válidos.
        if (data.indexOf(NOT_FOUND_MARK) == -1) {

          if (searchEngines[i][DATA_SERVER_PRIORITY] > lastPriority) { return true};

          data = getDataBetweenTags(data, searchEngines[i][DATA_HTML_TAG_BEGIN], searchEngines[i][DATA_HTML_TAG_END], searchEngines[i][DATA_HTML_TAG_OCURRENCE]);

          if (data != '') {
            //saveTextFile(Amarok.Info.scriptPath(), 'goo.html', data);
            //Amarok.Window.Statusbar.shortMessage(searchEngines[i][DATA_FOUND_STRING]);
            Amarok.Window.Statusbar.shortMessage(TEXT_MSG_FOUND);
            lastPriority = searchEngines[i][DATA_SERVER_PRIORITY];
            saveLyrics(formatHTML(initial_artist, initial_title, searchGlobals[i][GLOBALS_ARTIST], searchGlobals[i][GLOBALS_LYRIC], searchEngines[i][DATA_SERVER_NAME], searchEngines[i][DATA_SERVER_URL], html2text(data)));

            // DEBUG
            if (searchGlobals[SEARCH_ENGINE_ALTERNATIVES][GLOBALS_ARTIST] != '') {
              Amarok.Window.Statusbar.longMessage('Using alternative "' + searchGlobals[SEARCH_ENGINE_ALTERNATIVES][GLOBALS_ARTIST] + '"');
            }
            // DEBUG

            return true;
          }

        } // if (data.indexOf(NOT_FOUND_MARK) == -1)

        // TODO: Bug de sincronización entre threads. Debo buscar una solución a este problema.
        if ((hitsCount == SEARCH_ENGINES) && (lastPriority == LOWEST_PRIORITY)) {

          // Búsqueda de alternativas.
          // Vamos a desactivar esta feature.
          /*
          if (searchGlobals[SEARCH_ENGINE_ALTERNATIVES][GLOBALS_ARTIST] == '') {

            searchGlobals[SEARCH_ENGINE_ALTERNATIVES][GLOBALS_ARTIST] = getEquivalence(global_artist);
            if (searchGlobals[SEARCH_ENGINE_ALTERNATIVES][GLOBALS_ARTIST] == '') {
              searchGlobals[SEARCH_ENGINE_ALTERNATIVES][GLOBALS_ARTIST] == NOT_FOUND_MARK;
            } else {
              getLyrics(searchGlobals[SEARCH_ENGINE_ALTERNATIVES][GLOBALS_ARTIST], global_title, FLAG_RETRY);
              return true;
            }

          } // if (searchGlobals[SEARCH_ENGINE_ALTERNATIVES][GLOBALS_ARTIST] == '')
          */

          var repeatSearch = cfg[CFG_REPEAT_SEARCH];
          if (repeatSearch && cfg[CFG_REPEAT_SEARCH_INCOLLECTION_ONLY]) {
            repeatSearch = Amarok.Engine.currentTrack().inCollection;
          }

          if (repeatSearch) {

            lyricsSearch(global_title, global_artist);

          } else {

            Amarok.Window.Statusbar.shortMessage(TEXT_MSG_NOT_FOUND);
            if (html2text(Amarok.Engine.currentTrack().lyrics) == '') {
              saveLyrics(formatHTML(initial_artist, initial_title, '', '', '', '', TEXT_SONG_NOT_FOUND), true);
            }

          } // if (repeatSearch)

          /*
          var newData = null;
          if (cfg[CFG_REPEAT_SEARCH]) { newData = titleArtistInputDialog(TEXT_MSG_TRY_AGAIN, global_artist, global_title); };

          if (newData == null) {

            if (html2text(Amarok.Engine.currentTrack().lyrics) == '') {
              Amarok.Window.Statusbar.shortMessage(TEXT_MSG_NOT_FOUND);
              saveLyrics(formatHTML(initial_artist, initial_title, '', '', '', '', TEXT_SONG_NOT_FOUND), true);
            }

          } else {

            if (newData == '') {
              // Instrumental.
              Amarok.Window.Statusbar.shortMessage(TEXT_MSG_NOT_EXISTS);
              saveLyrics(formatHTML(initial_artist, initial_title, '', '', '', '', TEXT_SONG_INSTRUMENTAL));
            } else {
              newTitle = trim(getDataBetweenTags(newData, '"', '" - "', 1));
              newArtist = trim(getDataBetweenTags(newData, '" - "', '"', 1));
              getLyrics(newArtist, newTitle, FLAG_RETRY);
            } // if (newData == '')

          } // if (newData == null)
          */

        } // if ((hitsCount == SEARCH_ENGINES) && (lastPriority == 99))

        return true;

      } // if (data.indexOf(searchEngines[i, 2]) >= )
    } // for (var i = 0; i <= SEARCH_ENGINES; i++)

  } catch(err) {
        Amarok.debug( "error: " + err );
  }

} // function onDownloadFinished(data)


//
// Función principal.
//
function getLyrics(artist, title, url, ignoreCache) {
  try {

    //Amarok.alert(artist + '<>' + title + '<>' + url);

    var ignoreCache = (ignoreCache == null) ? false : ignoreCache;

    // Inicializamos variables globales.
    if (url != FLAG_RETRY) {
      // Puede venir de askForLyrics así que no nos podemos fiar
      // de los parámetros artist y title.
      initial_artist = trim(Amarok.Engine.currentTrack().artist);
      initial_title = trim(Amarok.Engine.currentTrack().title);
      searchGlobals[SEARCH_ENGINE_ALTERNATIVES] = new Array('', '');
    } else {
      // Para que no interfiera más abajo FLAG_RETRY se debe
      // interpretar como si se pasara una url vacía.
      url = '';
    }

    global_artist = artist;
    global_title = title;
    searchGlobals[SEARCH_ENGINE_GOO] = new Array('', '');
    searchGlobals[SEARCH_ENGINE_CIX] = new Array('', '');
    searchGlobals[SEARCH_ENGINE_LYRICWIKI] = new Array('', '');
    searchGlobals[SEARCH_ENGINE_SOGOU] = new Array('', '');
    searchGlobals[SEARCH_ENGINE_EVESTA] = new Array('', '');

    // Estas variables gestionan la interacción entre los servidores
    // y en un futuro la precedencia entre ellos.
    lastPriority = LOWEST_PRIORITY;
    hitsCount = 0;

    if (url != '') {

      Amarok.Window.Statusbar.shortMessage(TEXT_MSG_REUSING);
      var html = Amarok.Engine.currentTrack().lyrics;
      Amarok.Lyrics.showLyricsHtml(html);
      if (cfg[CFG_EXTERNAL_LYRICS_SHOW]) {saveToSTDIN(html)};

    } else {

      // Si existe en la cache lo reutilizamos.
      var html = '';
      if (!ignoreCache) { html = readTextFile(Amarok.Info.scriptPath() + '/' + PATH_LYRICS + '/' + getNameExternalFileTrack('html')); }

      if (html != '') {

        Amarok.Window.Statusbar.shortMessage(TEXT_MSG_USING_CACHE);
        saveLyrics(html, true); // (lyrics, ignoreCache)

      } else {

        Amarok.Window.Statusbar.shortMessage(TEXT_MSG_SEARCHING);
        search_lyricWiki(artist, title);
        search_goo(artist, title);
        search_cix(artist, title);
        search_sogou(artist, title);
        search_evesta(artist, title);

      } // if (html != '')

    } // if (url != '')

  } catch(err) {
    Amarok.debug( "error: " + err );
   }

} // function getLyrics(artist, title, url)

function askForLyrics() {

  // Si amarok no está tocando no seguimos.
  if (Amarok.Engine.engineState() == AMAROK_ENGINE_STATE_STOP) {
    Amarok.Window.Statusbar.longMessage(TEXT_MSG_OPTION_DISABLED);
    return true;
  }

  var artist = Amarok.Engine.currentTrack().artist;
  var title = Amarok.Engine.currentTrack().title;

  lyricsSearch(title, artist);

  return true;

} // function askForLyrics()

function askForLyricsAgain() {

  // Si amarok no está tocando no seguimos.
  if (Amarok.Engine.engineState() == AMAROK_ENGINE_STATE_STOP) {
    Amarok.Window.Statusbar.longMessage(TEXT_MSG_OPTION_DISABLED);
    return true;
  }

  var artist = Amarok.Engine.currentTrack().artist;
  var title = Amarok.Engine.currentTrack().title;

  lyricsSearch(title, artist, true);

  return true;

} // function askForLyricsAgain()

function showLyrics() {

  // Si amarok no está tocando no seguimos.
  if (Amarok.Engine.engineState() == AMAROK_ENGINE_STATE_STOP) {
    Amarok.Window.Statusbar.longMessage(TEXT_MSG_OPTION_DISABLED);
    return true;
  }

  var fileName = getNameExternalFileTrack('html');
  var convertedData = readTextFile(Amarok.Info.scriptPath() + '/' + PATH_LYRICS + '/' + fileName);
  if (convertedData != '') {

    var te = new textEditor(convertedData, fileName, 'L', '"' + Amarok.Engine.currentTrack().title + '" ' + TEXT_HTML_BY + ' "' + Amarok.Engine.currentTrack().artist + '"');
    te.btnReload.clicked.connect(askForLyricsAgain);
    te.show();

  } else {

    //TODO: lo ideal sería que la form se abriera al acabar esto.
    askForLyrics();

  } // if (convertedData != '')

  return true;

} // function showLyrics()

//
// Constantes.
//

// Array de searchEngines.
const SEARCH_ENGINES = 5;
const SEARCH_ENGINE_GOO = 0;
const SEARCH_ENGINE_CIX = 1;
const SEARCH_ENGINE_LYRICWIKI = 2;
const SEARCH_ENGINE_SOGOU = 3;
const SEARCH_ENGINE_EVESTA = 4;
const SEARCH_ENGINE_ALTERNATIVES = 5 // Es especial y siempre debe ser el último.

// Array de engine, un elemento del array de searchEngines.
const DATA_SERVER_URL = 0;
const DATA_SERVER_SEARCH = 1;
const DATA_SERVER_MARK = 2;
const DATA_FOUND_STRING = 3;
const DATA_HTML_TAG_BEGIN = 4;
const DATA_HTML_TAG_END = 5;
const DATA_HTML_TAG_OCURRENCE = 6;
const DATA_SERVER_NAME = 7;
const DATA_SERVER_PRIORITY = 8;

// Array de searchGlobals.
const GLOBALS_LYRIC = 0;
const GLOBALS_ARTIST = 1;
const GLOBALS_LYRIC_EQ = 2;
const GLOBALS_ARTIST_EQ = 3;

// QDir.exists() y QDir.mkdir() no funcionan.
var scriptPath = new QDir(Amarok.Info.scriptPath());
if (!scriptPath.exists(PATH_CACHE)) { scriptPath.mkdir(PATH_CACHE); }
if (!scriptPath.exists(PATH_LYRICS)) { scriptPath.mkdir(PATH_LYRICS); }
if (!scriptPath.exists(PATH_KANA)) { scriptPath.mkdir(PATH_KANA); }
if (!scriptPath.exists(PATH_KOREAN)) { scriptPath.mkdir(PATH_KOREAN); }
if (!scriptPath.exists(PATH_ROMAJA)) { scriptPath.mkdir(PATH_ROMAJA); }
if (!scriptPath.exists(PATH_ROMAJI)) { scriptPath.mkdir(PATH_ROMAJI); }
if (!scriptPath.exists(PATH_TRANSLATIONS)) { scriptPath.mkdir(PATH_TRANSLATIONS); }

// Otras constantes.
const EQUIVALENCES_FILE_NAME = 'equivalences.txt';
const FLAG_RETRY = '<<RETRY>>';
const LOWEST_PRIORITY = 999;
const NOT_FOUND_MARK = '<pre>Not found</pre>';
const GOOGLE_PACKET_MARK_SIZE = 5;
const GOOGLE_GET_METHOD_DATA_SIZE = 16;

// Esto debería ir grabado en el fichero de configuración.
var searchEngines = new Array();
searchEngines[SEARCH_ENGINE_GOO] = new Array('http://www.goo.ne.jp/', 'http://music.goo.ne.jp', 'goo.ne.jp', 'GOO lyrics found :)', '<p class="fs16">', '</p>', 2, 'goo 音楽', 1);
searchEngines[SEARCH_ENGINE_CIX] = new Array('http://www.cix.co.kr/', 'http://www.cix.co.kr', 'cix.co.kr', 'CIX lyrics found :)', '<!-- google_ad_section_start-->', '<!-- google_ad_section_end-->', 1, 'CIX Lyrics', 3);
searchEngines[SEARCH_ENGINE_LYRICWIKI] = new Array('http://lyricwiki.org/', 'http://lyricwiki.org', 'lyricwiki.org', 'LyricWiki lyrics found :)', '<pre>', '</pre>', 0, 'LyricWiki.org', 4);
searchEngines[SEARCH_ENGINE_SOGOU] = new Array('http://mp3.sogou.com/', 'http://mp3.sogou.com', 'mp3.sogou.com', 'Sogou lyrics found :)', '<td rowspan="2" style="line-height:21px"><font class=mr>', '</font>', 1, 'Sogou', 5);
searchEngines[SEARCH_ENGINE_EVESTA] = new Array('http://www.evesta.jp/', 'http://www.evesta.jp', 'www.evesta.jp', 'evesta lyrics found :)', '<p id=\'lyric\'>', '</p>', 1, 'evesta.jp', 2);

// Variables globales.
var searchGlobals = new Array();
searchGlobals[SEARCH_ENGINE_GOO] = '';
searchGlobals[SEARCH_ENGINE_CIX] = '';
searchGlobals[SEARCH_ENGINE_LYRICWIKI] = '';
searchGlobals[SEARCH_ENGINE_SOGOU] = '';
searchGlobals[SEARCH_ENGINE_EVESTA] = '';
searchGlobals[SEARCH_ENGINE_ALTERNATIVES] = '';

var initial_artist = '';
var initial_title = '';
var global_artist = '';
var global_title = '';
var lastPriority = LOWEST_PRIORITY;
var hitsCount = 0;

// Leemos las equivalencias.
loadEquivalences();

// Añadimos el menú de configuración.
Amarok.Window.addSettingsSeparator();
Amarok.Window.addSettingsMenu('LWS_script_configure', TEXT_MENU_CFG, 'configure');
var menuSettingsItem1 = Amarok.Window.SettingsMenu.LWS_script_configure;
menuSettingsItem1['triggered()'].connect(configureDialog);

// Añadimos el menú de pedir datos.
Amarok.Window.addToolsSeparator();
Amarok.Window.addToolsMenu('LWS_ask_for_lyrics', TEXT_MENU_GET_LYRICS, Amarok.Info.scriptPath() + '/icons/get_lyrics.png');
var menuToolsItem11 = Amarok.Window.ToolsMenu.LWS_ask_for_lyrics;
menuToolsItem11['triggered()'].connect(askForLyrics);

Amarok.Window.addToolsMenu('LWS_show_lyrics', TEXT_MENU_SHOW_LYRICS, Amarok.Info.scriptPath() + '/icons/lyrics.png');
var menuToolsItem12 = Amarok.Window.ToolsMenu.LWS_show_lyrics;
menuToolsItem12['triggered()'].connect(showLyrics);

// Gestión de equivalencias.
Amarok.Window.addToolsSeparator();
Amarok.Window.addToolsMenu('LWS_edit_equivalences', TEXT_MENU_EDIT_EQUIVALENCES, Amarok.Info.scriptPath() + '/icons/edit_db.png');
var menuToolsItem21 = Amarok.Window.ToolsMenu.LWS_edit_equivalences;
menuToolsItem21['triggered()'].connect(editEquivalences);

// Translation utilities.
Amarok.Window.addToolsSeparator();
Amarok.Window.addToolsMenu('LWS_translate', TEXT_MENU_TRANSLATE, Amarok.Info.scriptPath() + '/icons/google.png');
var menuToolsItem31 = Amarok.Window.ToolsMenu.LWS_translate;
menuToolsItem31['triggered()'].connect(translate);

// Japanese utilities.
Amarok.Window.addToolsSeparator();
Amarok.Window.addToolsMenu('LWS_to_kana', TEXT_MENU_TO_KANA, Amarok.Info.scriptPath() + '/icons/jp.png');
var menuToolsItem41 = Amarok.Window.ToolsMenu.LWS_to_kana;
menuToolsItem41['triggered()'].connect(toKana);

Amarok.Window.addToolsMenu('LWS_to_romaji', TEXT_MENU_TO_ROMAJI, Amarok.Info.scriptPath() + '/icons/jp.png');
var menuToolsItem42 = Amarok.Window.ToolsMenu.LWS_to_romaji;
menuToolsItem42['triggered()'].connect(toRomaji);
/*
Amarok.Window.addToolsMenu('LWS_to_korean', TEXT_MENU_TO_KOREAN, Amarok.Info.scriptPath() + '/icons/jp.png');
var menuToolsItem43 = Amarok.Window.ToolsMenu.LWS_to_korean;
menuToolsItem43['triggered()'].connect(toKorean);

Amarok.Window.addToolsMenu('LWS_to_arabic', TEXT_MENU_TO_ARABIC, Amarok.Info.scriptPath() + '/icons/jp.png');
var menuToolsItem44 = Amarok.Window.ToolsMenu.LWS_to_arabic;
menuToolsItem44['triggered()'].connect(toArabic);

Amarok.Window.addToolsMenu('LWS_to_russian', TEXT_MENU_TO_RUSSIAN, Amarok.Info.scriptPath() + '/icons/jp.png');
var menuToolsItem45 = Amarok.Window.ToolsMenu.LWS_to_russian;
menuToolsItem45['triggered()'].connect(toRussian);
*/
// Korean utilities.
Amarok.Window.addToolsSeparator();
Amarok.Window.addToolsMenu('LWS_to_romaja', TEXT_MENU_TO_ROMAJA, Amarok.Info.scriptPath() + '/icons/kr.png');
var menuToolsItem51 = Amarok.Window.ToolsMenu.LWS_to_romaja;
menuToolsItem51['triggered()'].connect(toRomaja);

//Amarok.Engine.trackChanged.connect(getLyrics);
Amarok.Lyrics.fetchLyrics.connect(getLyrics);

