/**************************************************************************
*   Amarok 2 script to manage lyrics                                      *
*                                                                         *
*   Copyright                                                             *
*   (C) 2008 Ignacio Serantes <kde@aynoa.net>                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.         *
**************************************************************************/

function searchDialog(title, artist, ignoreCache) {

  this.title = (title == null) ? Amarok.Engine.currentTrack().title : title;
  this.artist = (artist == null) ? Amarok.Engine.currentTrack().artist : artist;

  this.accept = function() {

    // Si amarok no está tocando no seguimos.
    if (Amarok.Engine.engineState() == AMAROK_ENGINE_STATE_STOP) {
      Amarok.Window.Statusbar.longMessage(TEXT_MSG_OPTION_DISABLED);
      return true;
    }

    if (this.ckInstrumental.checked) {

      // Instrumental.
      Amarok.Window.Statusbar.shortMessage(TEXT_MSG_NOT_EXISTS);
      saveLyrics(formatHTML(this.cbArtist.currentText, this.leTitle.text, '', '', '', '', TEXT_SONG_INSTRUMENTAL));

    } else {

      getLyrics(this.cbArtist.currentText, this.leTitle.text, '', this.ckIgnoreCache.checked);

    } // if (instrumental)

    this.close();

    return true;

  } // this.accept = function()

  this.cancel = function() {

    // Si amarok no está tocando no seguimos.
    if (Amarok.Engine.engineState() == AMAROK_ENGINE_STATE_STOP) {
      Amarok.Window.Statusbar.longMessage(TEXT_MSG_OPTION_DISABLED);
      return true;
    }

    if ((Amarok.Engine.currentTrack().inCollection) && (html2text(Amarok.Engine.currentTrack().lyrics) == '')) {

      Amarok.Window.Statusbar.shortMessage(TEXT_MSG_NOT_FOUND);
      saveLyrics(formatHTML(Amarok.Engine.currentTrack().artist, Amarok.Engine.currentTrack().title, '', '', '', '', TEXT_SONG_NOT_FOUND), true);

    } // if ((Amarok.Engine.currentTrack().inCollection) && (html2text(Amarok.Engine.currentTrack().lyrics) == ''))

    this.close();

    return true;

  } // this.cancel = function()

  this.show = function() {
    this.dlg.show();
  }

  this.close = function() {
    this.dlg.close();
  }

  this.dlg = new QWidget(this);

  // Fields group.
  this.grFields = new QGroupBox('', this.dlg);
  this.glFields = new QGridLayout(this.dlg);

  this.lbTitle = new QLabel(TEXT_SEARCH_DIALOG_LBTITLE, this.dlg);
  this.leTitle = new QLineEdit(this.dlg);
  this.ckInstrumental = new QCheckBox(TEXT_SEARCH_DIALOG_CBINSTRUMENTAL, this.dlg);

  this.lbArtist = new QLabel(TEXT_SEARCH_DIALOG_LBARTIST, this.dlg);
  this.cbArtist = new QComboBox(this.dlg);
  this.cbArtist.editable = true;
  this.ckIgnoreCache = new QCheckBox(TEXT_SEARCH_DIALOG_CBIGNORECACHE, this.dlg);

  this.lbServer = new QLabel(TEXT_SEARCH_DIALOG_LBSERVER, this.dlg);
  this.cbServer = new QComboBox(this.dlg);

  this.glFields.addWidget(this.lbTitle, 10, 0);
  this.glFields.addWidget(this.lbArtist, 20, 0);
  this.glFields.addWidget(this.lbServer, 30, 0);
  this.glFields.addWidget(this.leTitle, 10, 20);
  this.glFields.addWidget(this.cbArtist, 20, 20);
  this.glFields.addWidget(this.cbServer, 30, 20);
  this.glFields.addWidget(this.ckInstrumental, 10, 30);
  this.glFields.addWidget(this.ckIgnoreCache, 20, 30);
  this.grFields.setLayout(this.glFields);

  // Buttons group.
  this.hblButtons = new QHBoxLayout();
  this.hblButtons.addStretch();
  this.btOk = new QPushButton(new QIcon(Amarok.Info.scriptPath() + '/icons/ok.png'), TEXT_SEARCH_DIALOG_BTOK, this.dlg);
  this.btOk.clicked.connect(this, this.accept);
  this.btCancel = new QPushButton(new QIcon(Amarok.Info.scriptPath() + '/icons/cancel.png'), TEXT_SEARCH_DIALOG_BTCANCEL, this.dlg);
  this.btCancel.clicked.connect(this, this.cancel);
  this.hblButtons.addWidget(this.btOk, 0, 0);
  this.hblButtons.addWidget(this.btCancel, 0, 0);

  // Main layout.
  this.vblMain = new QVBoxLayout(this.dlg);
  this.vblMain.addWidget(this.grFields, 0, 0);
  this.vblMain.addStretch();
  this.vblMain.addLayout(this.hblButtons);

  this.dlg.setLayout(this.vblMain);
  this.dlg.sizeHint = new QSize(450, 150);
  this.dlg.size = new QSize(450, 150);

  this.dlg.setWindowTitle(TEXT_SCRIPT_NAME + ' - ' + TEXT_SEARCH_DIALOG_TITLE);

  // Añadimos los datos.
  this.leTitle.setText(title);
  this.cbArtist.addItem(artist);
  this.cbArtist.addItems(getEquivalences(artist));

  this.cbServer.addItem(TEXT_SERVER_AUTOMATIC);
  this.cbServer.enabled = false;

  // Nos aseguramos que la ventana se cierra al cambiar de tema o al parar de reproducir.
  Amarok.Engine.trackFinished.connect(this, this.close);
  Amarok.Engine.trackChanged.connect(this, this.close);

  this.ckIgnoreCache.checked = (ignoreCache == null) ? false : ignoreCache;

  this.show();

} // function searchDialog

function lyricsSearch(title, artist, ignoreCache) {

  var sd = new searchDialog(title, artist, ignoreCache);
  return true;

} // function searchDialog(title, artist)
