/**************************************************************************
*   Amarok 2 script to manage lyrics                                      *
*                                                                         *
*   Copyright                                                             *
*   (C) 2008 Ignacio Serantes <kde@aynoa.net>                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.         *
**************************************************************************/

function textEditor(text, fileName, tipo, subtitle) {

  this.fileName = (fileName == null) ? '' : fileName;
  this.subtitle = (subtitle == null) ? '' : ' - ' + subtitle;
  this.text = text;
  this.tipo = (tipo == null) ? '' : tipo;
  this.plainText = false;

  this.show = function() {
    this.dlg.show();
  }

  this.close = function() {
    this.dlg.close();
  }

  this.textChanged = function() {
    if (this.btnSaveText != '') { this.btnSave.enabled = true; }
  }

  this.saveFile = function() {

    var data = this.tdText.html;
    if (this.plainText) { data = html2text(data); }
    if (this.tipo == 'E') {
      saveAndLoadEquivalences(this, data);
    } else {
      saveTextFile(Amarok.Info.scriptPath() + '/' + this.path, this.fileName, data);
    }

    this.btnSave.enabled = false;
    this.btnDelete.enabled = true;

  } // this.saveFile = function()

  this.deleteFile = function() {

    if (this.tipo != 'E') {

      var file = new QFile(Amarok.Info.scriptPath() + '/' + this.path + '/' + this.fileName);
      file.remove();
      if (this.tipo != 'L') { this.btnSave.enabled = false; }
      this.btnDelete.enabled = false;
      this.close()

    } // if (this.tipo != 'E')

  } // this.deleteFile = function()

  this.dlg = new QWidget(this);

  // Buttons layout.
  this.hlButtons = new QHBoxLayout();
  this.hlButtons.addStretch();

  switch(this.tipo) {

    case 'E':
      this.btnReloadText = '';
      this.btnSaveText = TEXT_TEXT_DIALOG_BTSAVE_EQUIVALENCES;
      this.path = '';
      this.plainText = true;
      this.tipoTitle = '';
      break;

    case 'H':
      this.btnReloadText = TEXT_TEXT_DIALOG_BTRELOAD_ROMAJA;
      this.btnSaveText = (cfg[CFG_CACHE_ROMAJA]) ? TEXT_TEXT_DIALOG_BTSAVE_ROMAJA : '';
      this.path = PATH_ROMAJA;
      this.plainText = false;
      this.tipoTitle = TEXT_TEXT_DIALOG_SUBTITLE_ROMAJA;
      break;

    case 'K':
      this.btnReloadText = TEXT_TEXT_DIALOG_BTRELOAD_KANA;
      this.btnSaveText = (cfg[CFG_CACHE_KANA]) ? TEXT_TEXT_DIALOG_BTSAVE_KANA : '';
      this.path = PATH_KANA;
      this.plainText = false;
      this.tipoTitle = TEXT_TEXT_DIALOG_SUBTITLE_KANA;
      break;

    case 'L':
      this.btnReloadText = TEXT_TEXT_DIALOG_BTRELOAD_LYRICS;
      this.btnSaveText = (cfg[CFG_CACHE_LYRICS]) ? TEXT_TEXT_DIALOG_BTSAVE_LYRICS : '';
      this.path = PATH_LYRICS;
      this.plainText = false;
      this.tipoTitle = TEXT_TEXT_DIALOG_SUBTITLE_LYRICS;
      break;

    case 'O':
      this.btnReloadText = TEXT_TEXT_DIALOG_BTRELOAD_KOREAN;
      this.btnSaveText = (cfg[CFG_CACHE_KOREAN]) ? TEXT_TEXT_DIALOG_BTSAVE_KOREAN : '';
      this.path = PATH_KOREAN;
      this.plainText = false;
      this.tipoTitle = TEXT_TEXT_DIALOG_SUBTITLE_KOREAN;
      break;

    case 'R':
      this.btnReloadText = TEXT_TEXT_DIALOG_BTRELOAD_ROMAJI;
      this.btnSaveText = (cfg[CFG_CACHE_ROMAJI]) ? TEXT_TEXT_DIALOG_BTSAVE_ROMAJI : '';
      this.path = PATH_ROMAJI;
      this.plainText = false;
      this.tipoTitle = TEXT_TEXT_DIALOG_SUBTITLE_ROMAJI;
      break;

    case 'T':
      this.btnReloadText = TEXT_TEXT_DIALOG_BTRELOAD_TRANSLATION;
      this.btnSaveText = (cfg[CFG_CACHE_TRANSLATION]) ? TEXT_TEXT_DIALOG_BTSAVE_TRANSLATION : '';
      this.path = PATH_TRANSLATIONS;
      this.plainText = false;
      this.tipoTitle = TEXT_TEXT_DIALOG_SUBTITLE_TRANSLATION;
      break;

    default:
      this.btnSaveText = '';
      this.btnReloadText = '';
      this.tipoTitle = '';

  } // switch(tipo)

  if (this.fileName == '') { this.saveEnabled = false; }

  if (this.btnSaveText != '') {

    if (this.tipo != 'L') {
      this.btnSave = new QPushButton(new QIcon(Amarok.Info.scriptPath() + '/icons/save.png'), this.btnSaveText, this.dlg);
      this.btnSave.enabled = false;
      this.btnSave.clicked.connect(this, this.saveFile);
      this.hlButtons.addWidget(this.btnSave, 0, 0);
    } // if (this.tipo != 'L')

    if (this.tipo != 'E') {
      this.btnDelete = new QPushButton(new QIcon(Amarok.Info.scriptPath() + '/icons/erase.png'), TEXT_TEXT_DIALOG_BTDELETE_CACHE, this.dlg);
      this.btnDelete.clicked.connect(this, this.deleteFile);
      this.hlButtons.addWidget(this.btnDelete, 0, 0);
    } // if (this.tipo != 'E')

  } // if (this.btnSaveText != '')

  if (this.btnReloadText != '') {
    this.btnReload = new QPushButton(new QIcon(Amarok.Info.scriptPath() + '/icons/reload.png'), this.btnReloadText, this.dlg);
    this.btnReload.clicked.connect(this.dlg.close);
    this.hlButtons.addWidget(this.btnReload, 0, 0);
  } // if (this.btnReloadText != '')

  this.subtitle += this.tipoTitle;

  this.btnClose = new QPushButton(new QIcon(Amarok.Info.scriptPath() + '/icons/close.png'),TEXT_TEXT_DIALOG_BTCLOSE, this.dlg);
  this.btnClose.clicked.connect(this.dlg.close);

  this.hlButtons.addWidget(this.btnClose, 0, 0);

  // Main layout.
  this.tdText = new QTextEdit(this.dlg);
  if (this.tipo == 'L') {this.tdText.readOnly = true; }
  this.tdText.setText(this.text);
  this.tdText.textChanged.connect(this, this.textChanged);
  this.mainLayout = new QVBoxLayout(this.dlg);
  this.mainLayout.addWidget(this.tdText, 0, 0);
  this.mainLayout.addLayout(this.hlButtons);

  this.dlg.setLayout(this.mainLayout);
  this.dlg.sizeHint = new QSize(500, 600);
  this.dlg.size = new QSize(500, 600);

  this.dlg.setWindowTitle(TEXT_SCRIPT_NAME + this.subtitle);

} // function textEditor (text, fileName, tipo)
